#!/usr/bin/perl -w
#
# check_ifstatus.pl - netsaint plugin 
# 
#
# Copyright (C) 2000 Christoph Kron
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# IMPORT INFORMATION
# this module requires the perl module Net::SNMP
# the latest version of Net::SNMP can be found on:
# http://search.cpan.org/search?module=Net::SNMP
# or at http://www.cpan.org
#
# this plugin produces long output
# in base/netsaint.h increase #define MAX_INPUT_BUFFER to 1024
# then recompile and install netsaint for the changes to take effect
#
# Report bugs to: ck@zet.net
#
# 11.01.2000 Version 1.0
# 14.01.2000 Version 1.0.1 removed STATE from output
# 21.01.2000 Version 1.0.2 brialarm hack - suppress CRITCAL alert for BRI
#			   interfaces
# 18.01.2000 Version 1.1   flap control introduced
#
use strict;

use Net::SNMP;
use Getopt::Long;
&Getopt::Long::config('auto_abbrev');


my $status;
my $TIMEOUT = 10;

my %ERRORS = ('UNKNOWN' , '-1',
              'OK' , '0',
              'WARNING', '1',
              'CRITICAL', '2');

my %ifOperStatus = 	('1','up',
			 '2','down',
			 '3','testing',
			 '4','unknown',
			 '5','dormant',
			 '6','notPresent');

my $state = "UNKNOWN";
my $answer = "";
my $snmpkey;
my $snmpoid;
my $key;
my $community = "public";
my $port = 161;
my @snmpoids;
my $snmpIfAdminStatus = '1.3.6.1.2.1.2.2.1.7';
my $snmpIfDescr = '1.3.6.1.2.1.2.2.1.2';
my $snmpIfOperStatus = '1.3.6.1.2.1.2.2.1.8';
my $snmpLocIfDescr = '1.3.6.1.4.1.9.2.2.1.1.28';
my $hostname;
my $brialarm = 0;	# 0 disables critical alert for BRI interfaces
my $bridown =0;		
my $withflap=0;		# withhold critical alert when line is flapping
my $warnonly=0;		# no critical alert if warnonly eq 1
my $session;
my $error;
my $response;
my %ifStatus;
my $ifup =0 ;
my $ifdown =0;
my $ifdormant = 0;
my $ifmessage;
my $critical=0;
my $warning=0;
my $logdir="/tmp";
my %flapstatus;

sub usage {
  printf "\nMissing arguments!\n";
  printf "\n";
  printf "Perl Check IfStatus plugin for NetSaint\n";
  printf "monitors operational status of each interface\n";
  printf "usage: \n";
  printf "check_ifstatus.pl -b -c <READCOMMUNITY> -p <PORT> <HOSTNAME>\n";
  printf "-b\talways return CRITICAL alert for down BRI interfaces\n";
  printf "\tDefault is a WARNING alert for down BRI interfaces\n";
  printf "\tAny other interface type results in a CRITCAL alert\n";
  printf "<READCOMMUNITY> SNMP read community\n";
  printf "<PORT>\tUDP port for SNMP request\n";
  printf "<HOSTNAME> hostname or ip address\n\n";
  printf "Copyright (C) 2000 Christoph Kron\n";
  printf "check_ifstatus.pl comes with ABSOLUTELY NO WARRANTY\n";
  printf "This programm is licensed under the terms of the ";
  printf "GNU General Public License\n(check source code for details)\n";
  printf "\n\n";
  exit $ERRORS{"UNKNOWN"};
}


sub ReadLogfile {
  my ($logdir) = @_;
  my $logfile;
  my $index;
  my $downevent;
  my $upevent;
  my $flapstate;

  # if logdir does not exist interface state is down
  if ( ! -d $logdir )  { return(0); }

  $logfile = "$logdir" . "/" . "$hostname" . ".log";

  # if there is an existing logfile open it and read data
  if ( -r $logfile ) {
     if (open(LOGFILE,"<$logfile")) {
        while(<LOGFILE>) {
	   chomp();
	   ($index,$downevent,$upevent,$flapstate) =split(/\s/,$_,4);
	   $flapstatus{$index}{'DOWNEVENT'} = $downevent;
	   $flapstatus{$index}{'UPEVENT'} = $upevent;
	   $flapstatus{$index}{'FLAPSTATE'} = $flapstate;

	}
        close(LOGFILE);
     }
  }

}

sub WriteLogfile {
  my ($logdir) = @_;
  my $logfile;
  my $index;
  my $downevent;
  my $upevent;
  my $flapstate;

  # if logdir does not exist interface state is down
  if ( ! -d $logdir )   { return(0); }

  $logfile = "$logdir" . "/" . "$hostname" . ".log";

  # writing data to logfile
  if (open(LOGFILE,">$logfile")) {
     foreach $index (sort keys %flapstatus) {
	$downevent = $flapstatus{$index}{'DOWNEVENT'};
	$upevent = $flapstatus{$index}{'UPEVENT'};
	$flapstate = $flapstatus{$index}{'FLAPSTATE'};
        printf LOGFILE "%s\t%s\t%s\t%s\n",$index,$downevent,$upevent,$flapstate;;
     }
     close(LOGFILE);
     return(1);
  }
  else {
     return(0);
  }
}


# return values
# -1 error
# 0 down
# 1 up
# 3 flapping
#

sub FlapUp {
  my ($index,$flaptime) = @_;
  my $downevent;
  my $upevent;
  my $flapstate;
  my $now = time();

  if (!exists($flapstatus{$index})) {
     # initialize - no flap history
     $flapstatus{$index}{'DOWNEVENT'} = 0;
     $flapstatus{$index}{'UPEVENT'} = $now;
     $flapstatus{$index}{'FLAPSTATE'} = "up";
     return(1);
  }

  # if interface was already return flapstate = up
  if ($flapstatus{$index}{'FLAPSTATE'} eq "up" ) {
     return(1);
  }
     
  if ($flapstatus{$index}{'FLAPSTATE'} eq "flap") {
     # check if up event was within flaptime period
     if ($now - $flapstatus{$index}{'UPEVENT'} < $flaptime) {
        # still flapping
        return(3);
     }
     else {
        # 
        $flapstatus{$index}{'FLAPSTATE'} = "up";
	$flapstatus{$index}{'UPEVENT'} = $now;
	return(1);
     }
  }
  else {
     # FLAPSTATE == down but interface is up
     if ($now - $flapstatus{$index}{'DOWNEVENT'} > $flaptime) {
        # line not flapping 
        $flapstatus{$index}{'FLAPSTATE'} = "up";
	$flapstatus{$index}{'UPEVENT'} = $now;
	return(1);
     }
     else {
        # assume the line is flapping although interface is up
        $flapstatus{$index}{'FLAPSTATE'} = "flap";
	$flapstatus{$index}{'UPEVENT'} = $now;
	return(3);
     }   
  }
}

sub FlapDown {
  my ($index,$flaptime) = @_;
  my $downevent;
  my $upevent;
  my $flapstate;
  my $now = time();

  if (!exists($flapstatus{$index})) {
     # initialize - no flap history
     $flapstatus{$index}{'DOWNEVENT'} = $now;
     $flapstatus{$index}{'UPEVENT'} = 0;
     $flapstatus{$index}{'FLAPSTATE'} = "down";
     return(2);
  }

  # if interface was already down return flapstate = down
  if ($flapstatus{$index}{'FLAPSTATE'} eq "down" ) {
     return(2);
  }

  if ($flapstatus{$index}{'FLAPSTATE'} eq "flap") {
     # interface still down , no need to change FLAPSTATE
     return(3);
  }
  else {
     # FLAPSTATE == up but interface is down
     if ($now - $flapstatus{$index}{'UPEVENT'} > $flaptime) {
        # line not flapping 
        $flapstatus{$index}{'FLAPSTATE'} = "down";
	$flapstatus{$index}{'DOWNEVENT'} = $now;
	return(2);
     }
     else {
        # assume the line is flapping 
        $flapstatus{$index}{'FLAPSTATE'} = "flap";
	$flapstatus{$index}{'DOWNEVENT'} = $now;
	return(3);
     }   
  }

}

sub FlapControl {
  my ($key,$interface,$ifstate,$flaptime) = @_;

  my $status;
  my $index;

  # unique Key
  $index = "$key-$interface";
  # 1 -> up 2-> down
  if ($ifstate == 1) {
     $status = FlapUp($index,$flaptime);
  }
  elsif ($ifstate == 2) {
     $status = FlapDown($index,$flaptime);
  }
  else {
     $status = -1;
  }
  return($status);
}



# Just in case of problems, let's not hang NetSaint
$SIG{'ALRM'} = sub {
     print ("ERROR: No snmp response from $hostname (alarm)\n");
     exit $ERRORS{"UNKNOWN"};
};
alarm($TIMEOUT);


$status = GetOptions("community=s",\$community,
		     "brialarm",\$brialarm,
                     "port=i",\$port);
if ($status == 0)
{
        &usage;
}
  
   #shift;
   $hostname  = shift || &usage;

ReadLogfile("/tmp");

   push(@snmpoids,$snmpIfOperStatus);
   push(@snmpoids,$snmpLocIfDescr);
   push(@snmpoids,$snmpIfAdminStatus);
   push(@snmpoids,$snmpIfDescr);

foreach $snmpoid (@snmpoids) {

   ($session, $error) = Net::SNMP->session(
      -hostname  => $hostname,
      -community => $community,
      -port      => $port
   );

   if (!defined($session)) {
      $state='UNKNOWN';
      $answer=$error;
      print ("$answer");
      exit $ERRORS{$state};
   }

   if (!defined($response = $session->get_table($snmpoid))) {
      $answer=$session->error;
      $session->close;
      $state = 'UNKOWN';
      print ("$answer,$community\n");
      exit $ERRORS{$state};
   }

   foreach $snmpkey (keys %{$response}) {
      $snmpkey =~ /.*\.(\d+)$/;
      $key = $1;
      $ifStatus{$key}{$snmpoid} = $response->{$snmpkey};
   }
   $session->close;
}

   foreach $key (keys %ifStatus) {
      # check only if interface is administratively up
      if ($ifStatus{$key}{$snmpIfAdminStatus} == 1 ) {
         # interface with status == up
         if ($ifStatus{$key}{$snmpIfOperStatus} == 1 ) {
	    $ifup++;
	    if (FlapControl($key,$ifStatus{$key}{$snmpIfDescr},1,600) == 3) { 
	       # CRITICAL STATE because line is flapping
	       $critical++;
                $ifmessage .= sprintf("%s: flapping -> %s<BR>",
                                 $ifStatus{$key}{$snmpIfDescr},
				 $ifStatus{$key}{$snmpLocIfDescr});
	    }
	 }


         if ($ifStatus{$key}{$snmpIfOperStatus} == 2 ) {
             # interface with status == down
             $ifdown++ ;
	     if (FlapControl($key,$ifStatus{$key}{$snmpIfDescr},2,600) != 3) {
                $ifmessage .= sprintf("%s: down -> %s<BR>",
                                 $ifStatus{$key}{$snmpIfDescr},
				 $ifStatus{$key}{$snmpLocIfDescr});
	        # 64Kb or 128Kb leased line down
	        $critical++;
	        if (($ifStatus{$key}{$snmpIfDescr} =~ m/BRI/ && $brialarm == 0) 
		   || ($ifStatus{$key}{$snmpLocIfDescr} =~ m/<\s*WARNONLY\s*>/i)) 
		{
		   $critical--;
		   $warning++;
		}
              }
	      else {
	        $critical++;
	        if (($ifStatus{$key}{$snmpIfDescr} =~ m/BRI/ && $brialarm == 0) 
		   || ($ifStatus{$key}{$snmpLocIfDescr} =~ m/<\s*WARNONLY\s*>/i)) 
		{
		   $critical--;
		   $warning++;
		}
                $ifmessage .= sprintf("%s: flapping -> %s<BR>",
                                 $ifStatus{$key}{$snmpIfDescr},
				 $ifStatus{$key}{$snmpLocIfDescr});
	      }
         }
         if ($ifStatus{$key}{$snmpIfOperStatus} == 5 ) { $ifdormant++ ;}
      }
   }
   

   # decide which state returning to netsaint
   if ( $critical > 0 ) {
      $state = 'CRITICAL';
      $answer = sprintf("interfaces up: %d, down: %d, dormant: %d<BR>",
			$ifup,
			$ifdown,
			$ifdormant);
      $answer = $answer . $ifmessage . "\n";
   }
   elsif ($warning > 0){
      $state = 'WARNING';
      $answer = sprintf("interfaces up: %d, down: %d, dormant: %d<BR>",
			$ifup,
			$ifdown,
			$ifdormant);
      $answer = $answer . $ifmessage . "\n";
   }
   else {
      $state = 'OK';
      $answer = sprintf("interfaces up: %d, down: %d, dormant: %d\n",
			$ifup,
			$ifdown,
			$ifdormant);
   }

WriteLogfile("/tmp");
print ("$answer");
exit $ERRORS{$state};

